// ==========================================
// GAme using classes for the objects, monsters 
// and players
// ==========================================



// constants 
const FRAME_RATE = 30;     
const MONSTER_COUNT = 10; 


// global variables for drawing on the canvas
var gCanvas = document.getElementById("gameSpace");
var gCtx = gCanvas.getContext("2d");

// when the game was started, for the timer
gStartTime = null;

// global variables for lists of the objects on the screen
gObjectList = [];
gMonsterList = [];
gPlayerList = [];


// start the game
function startTheGame()
{
    // add static items to the game
    // for example:
    // gObjectList.push(new staticThing(80,130,gCtx));
    
    gObjectList.push(new staticThing(80,70,gCtx));
    gObjectList.push(new staticThing(80,100,gCtx));
    gObjectList.push(new staticThing(110,130,gCtx));


    
    // add players to the game
    // for example:
    // gPlayerList.push(new controllableMovingThing(350,350,gCtx,5,3,["ArrowUp","ArrowLeft","ArrowDown","ArrowRight"]))

    gPlayerList.push(new controllableMovingThing(20,20,gCtx,5,3,["w","a","s","d"]))

    // add monsters to the game all at the same 
    // point but moving in different directions
    for(var i=0; i< MONSTER_COUNT; i++)
    {
        gMonsterList.push(new movingThing(200,200,gCtx,5 - Math.random() * 10,5 - Math.random() * 10,));
    }


    // start the timer which calls the updater every 10th of a second
    gTimer = window.setInterval(updateGame,1000/FRAME_RATE);

    // start the keyboard listener
    addEventListener("keydown",handleKeyDown);

}


// when a key is pressed, decide what to do
function handleKeyDown(keyEvent)
{
    gPlayerList.forEach(player=>{
        player.reactToKeypress(keyEvent.key)
    })
}
// the main game loop. This function gets called for every frame of the game
function updateGame()
{

    // if no monster left, end the game
    if (gMonsterList.length == 0)
    {
        endTheGame();
    }

    // move the monsters - give the function a list of
    // all the static objects they have to not collide with
    gMonsterList.forEach(thing => {
        thing.move(gObjectList);
    });

    // similarly move the players
    gPlayerList.forEach(thing => {
        thing.move(gObjectList);
    });

    // detect if the players collided with a monster
    gPlayerList.forEach(player => {
        var monsterHit = player.collidesWithAnyOf(gMonsterList);
        // if a monster was hit, mark it for removal
        if (monsterHit != null) monsterHit.kill();
    });
    
    
    // draw the players and monsters
    drawGame();

    // cull any monsters to be killed.
    // why has this not used a for loop???
    var i=0;
    while(i<gMonsterList.length)
    {
        if (gMonsterList[i].toBeCulled())
        {
            gMonsterList.splice(i,1);
        }else{
            i++;
        }
    }



   
}



// erase the current display and draw all the objects in the game
function drawGame()
{

    // erase the background
    gCtx.fillStyle = "#FFFFFF";
    gCtx.fillRect(0,0,gCanvas.width,gCanvas.height);

    // draw the objects
    gObjectList.forEach(thing => {
        thing.draw();
    });

     // draw the players
    gPlayerList.forEach(thing => {
        thing.draw();
    });

    // draw the monsters
    gMonsterList.forEach(thing => {
        thing.draw();
    });
    
   

}



// stop the timer when the game ends
function endTheGame()
{
    clearInterval(gTimer);
    gObjectList = [];
    gPlayerList = [];
    gMonsterList = [];
}
